package com.sun.xml.internal.ws.policy;

import java.util.Iterator;
import java.util.Collection;
import java.util.Queue;
import java.util.List;
import com.sun.xml.internal.ws.policy.privateutil.LocalizationMessages;
import java.util.ArrayList;
import java.util.LinkedList;
import com.sun.xml.internal.ws.policy.sourcemodel.wspolicy.NamespaceVersion;
import com.sun.xml.internal.ws.policy.privateutil.PolicyLogger;

public final class PolicyIntersector
{
    private static final PolicyIntersector STRICT_INTERSECTOR;
    private static final PolicyIntersector LAX_INTERSECTOR;
    private static final PolicyLogger LOGGER;
    private PolicyIntersector.CompatibilityMode mode;
    
    private PolicyIntersector(PolicyIntersector.CompatibilityMode intersectionMode) {
        this.mode = intersectionMode;
    }
    
    public static PolicyIntersector createStrictPolicyIntersector() {
        return PolicyIntersector.STRICT_INTERSECTOR;
    }
    
    public static PolicyIntersector createLaxPolicyIntersector() {
        return PolicyIntersector.LAX_INTERSECTOR;
    }
    
    public Policy intersect(Policy... policies) {
        if (policies == null || policies.length == 0) {
            throw (IllegalArgumentException)PolicyIntersector.LOGGER.logSevereException(new IllegalArgumentException(LocalizationMessages.WSP_0056_NEITHER_NULL_NOR_EMPTY_POLICY_COLLECTION_EXPECTED()));
        }
        if (policies.length == 1) {
            return policies[0];
        }
        boolean found = false;
        boolean allPoliciesEmpty = true;
        NamespaceVersion latestVersion = null;
        int length = policies.length;
        int j = 0;
        while (j < length) {
            Policy tested = policies[j];
            if (tested.isEmpty()) {
                found = true;
            }
            else {
                if (tested.isNull()) {
                    found = true;
                }
                allPoliciesEmpty = false;
            }
            if (latestVersion == null) {
                latestVersion = tested.getNamespaceVersion();
            }
            else if (latestVersion.compareTo(tested.getNamespaceVersion()) < 0) {
                latestVersion = tested.getNamespaceVersion();
            }
            if (found && !allPoliciesEmpty) {
                return Policy.createNullPolicy(latestVersion, null, null);
            }
            ++j;
        }
        latestVersion = ((latestVersion != null) ? latestVersion : NamespaceVersion.getLatestVersion());
        if (allPoliciesEmpty) {
            return Policy.createEmptyPolicy(latestVersion, null, null);
        }
        List<AssertionSet> finalAlternatives = new LinkedList(policies[0].getContent());
        Queue<AssertionSet> testedAlternatives = new LinkedList();
        List<AssertionSet> alternativesToMerge = new ArrayList(2);
        for (int i = 1; i < policies.length; ++i) {
            Collection<AssertionSet> currentAlternatives = policies[i].getContent();
            testedAlternatives.clear();
            testedAlternatives.addAll(finalAlternatives);
            finalAlternatives.clear();
            AssertionSet testedAlternative;
            while ((testedAlternative = (AssertionSet)testedAlternatives.poll()) != null) {
                for (AssertionSet currentAlternative : currentAlternatives) {
                    if (testedAlternative.isCompatibleWith(currentAlternative, this.mode)) {
                        alternativesToMerge.add(testedAlternative);
                        alternativesToMerge.add(currentAlternative);
                        finalAlternatives.add(AssertionSet.createMergedAssertionSet(alternativesToMerge));
                        alternativesToMerge.clear();
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return Policy.createPolicy(latestVersion, null, null, finalAlternatives);
    }
    
    static {
        STRICT_INTERSECTOR = new PolicyIntersector(PolicyIntersector.CompatibilityMode.STRICT);
        LAX_INTERSECTOR = new PolicyIntersector(PolicyIntersector.CompatibilityMode.LAX);
        LOGGER = PolicyLogger.getLogger(PolicyIntersector.class);
    }
    
    enum CompatibilityMode
    {
        STRICT, 
        LAX;
    }
}
